@@ -4,7 +4,7 @@ c             hadron A collision by Lund
 
 #include  "Zptcl.h"
 #include  "Zcode.h"
-
+#include  "Zevhnv.h"
 c
         record /ptcl/ pj, a(*)
         integer ia,iz, np
@@ -19,7 +19,10 @@ c
 
          elab=pj.fm.p(4)
          ek = elab - pj.mass
-         if(elab .gt. 5. ) then
+c         if(elab .gt. 5. ) then
+         if(ActiveMdl .eq. 'fritiof1.6'
+     *    .or. ( ActiveMdl .eq. 'byenergy' .and.
+     *           ek .gt. 4.0 ) ) then
 c                lund monte carlo ; fritiof
             call cfritiof(pj, ia, iz,  a, np)
 c              in  above, momentum  is measured from pj.fm so 
@@ -28,11 +31,19 @@ c       ////////
 ccc            call ccheckMom('after fritio: before rot', a, np)
 c      /////////            
             call crot3mom(pj, a, np)
-         elseif(ek  .gt. eintmn  ) then
+         elseif(ek  .gt. eintmn .and.
+     *         ( ActiveMdl .eq. 'nucrin' .or.
+     *          ActiveMdl .eq. 'byenergy' ) ) then
             call cnucrin(pj, ia, iz,  a, np)
             call crot3mom(pj, a, np)
-         elseif(pj.code .eq. knuc .and. pj.subcode .eq. antip) then
-c              for anti-barion, no threshold.
+         elseif(pj.code .eq. knuc .and. pj.subcode .eq. antip
+     *      .and. (ActiveMdl .eq. 'nucrin'  .or.
+     *             ActiveMdl .eq. 'byenergy' ) ) then
+c              for anti-barion, no threshold.  completely 0
+c              K.E must be avoided for nucrin
+            if( pj.fm.p(4) - pj.mass .le. 0.0 ) then
+               pj.fm.p(4) = pj.mass*1.0001
+            endif
             call cnucrin(pj, ia, iz,  a, np)
             call crot3mom(pj, a, np)
          else
@@ -104,7 +115,7 @@ c
 
       integer mst
       real*4 par
-      character*90   msg
+      character*150   msg
 c ---------------------------------------------------------------------
 c              Debug-def    discard-def  notify-def use-strg-flg  halt
 c     debug       o             x           o           o          x
@@ -274,17 +285,19 @@ c        --- fix projectile --------------
                  iproty=18
              elseif(icg .eq. -1)then
                  iproty=-18
-             elseif(pj.subcode .eq. k0l) then
+             elseif(abs(pj.subcode) .eq. k0l) then
 c                  k0l
-                 iproty=38
-             elseif(pj.subcode .eq. k0s) then
+                 iproty=sign(38, pj.subcode)
+             elseif(abs(pj.subcode) .eq. k0s) then
 c                  k0s
-                 iproty=37
+                 iproty=sign(37, pj.subcode)
              endif
           else
              write(msg,*) ' ptcl code=',kidn,' not acceptable in ',
-     *                  ' cfritiof '
-             call cerrorMsg(msg, 0)
+     *                  ' cfritiof; event neglectd '
+             call cerrorMsg(msg, 1)
+             np = 0
+             return  !  exit
           endif
 #ifdef USE_STRANGEFLAG
 c           skip particles of undefined code; 
@@ -353,17 +366,16 @@ c                    n_b
                   ctmp=0
                   pap=antip
 c                -------------------  kaon  0
-              elseif(kk .eq. 37) then
-c          pap: -1--> k0 short, -2-->k0 long
-c                    kos
+              elseif(abs(kk) .eq. 37) then
+c                     kos
                   ktmp=kkaon
                   ctmp=0
-                  pap=k0s
-              elseif(kk .eq. 38) then
+                  pap=sign(k0s, kk)
+              elseif(abs(kk) .eq. 38) then
 c                    k0l
                   ktmp=kkaon
                   ctmp=0
-                  pap=k0l
+                  pap=sign(k0l, kk)
               elseif(kk .eq.  7) then
 c                    e-
                   ktmp=kelec
@@ -463,7 +475,7 @@ c
        real*4 elab, plab, anuc, znuc
        integer kidn, icg, iproty, itta, l, kk, ktmp, ctmp, pap
        real*8 et, u
-       character*70  msg
+       character*100  msg
 c
 c
          elab=pj.fm.p(4)
@@ -504,8 +516,10 @@ c        --- fix projectile --------------
              endif
           else
              write(msg,*) ' ptcl code=',kidn,' not acceptable in ',
-     *        ' cnucrin '
-             call cerrorMsg(msg, 0)
+     *        ' cnucrin; envet neglected '
+             call cerrorMsg(msg, 1)
+             np = 0
+             return  ! ***********
           endif
 c                generate ptcls
           if(ia .gt. 1) then
@@ -573,7 +587,7 @@ c                    n_b
                   ctmp=0.
                   pap=antip
 c                -------------------  k0
-              elseif(kk .eq. 24 .or. kk .eq. 25) then
+              elseif(kk .eq. 24) then
                   ktmp=kkaon
                   ctmp =0
                   call rndc(u)
@@ -582,6 +596,16 @@ c                -------------------  k0
                   else
                      pap = k0l
                   endif
+               elseif(kk .eq. 25) then
+c                     k0b
+                  ktmp=kkaon
+                  ctmp =0
+                  call rndc(u)
+                  if(u .lt. 0.5) then
+                     pap = -k0s
+                  else
+                     pap = -k0l
+                  endif
               elseif(kk .eq. 3)  then
 c                    e-
                   ktmp = kelec
@@ -603,9 +627,11 @@ c                    mu-
                   ctmp=-1
                   pap=regptcl
               else
-                  write(msg,*) ' ptcl code=',kk,' from nucrin',
+                 write(msg,*) ' ptcl code=',kk,' from nucrin',
      *            ' not acceptable in cnucrin'
-                  call cerrorMsg(msg, 0)
+                  call cerrorMsg(msg, 1)
+                  call cerrorMsg(' it is neglected' ,1)
+                  goto 10    ! *************
               endif
               np=np+1
               call cmkptc(ktmp, pap, ctmp, a(np))
@@ -620,6 +646,7 @@ c                  sometimes nucrin give unbalanced energy mom. at low E
      *        - a(np).fm.p(4)**2)/a(np).fm.p(4)  .gt. .005) then
                  call cadjm(a(np), a(np))
               endif
+ 10           continue
            enddo
         end
         subroutine chadrin(iproty, plab, elab, itta)
